@@ -7,7 +7,7 @@
 from typing import *
 import re
 
-from domonic.style import Style
+from domonic.style import CSSStyleDeclaration as Style
 import domonic.javascript
 from domonic.events import *
 
@@ -30,7 +30,6 @@ class EventTarget:
             return
 
         stack = self.listeners[_type]
-
         for thing in stack:
             if thing == callback:
                 stack.remove(thing)
@@ -472,9 +471,9 @@ class ChildNode(Node):
 
 class Attr(object):
 
-    def __init__(self, name, *args, **kwargs):
+    def __init__(self, name, value="", *args, **kwargs):
         self.name = name
-        self.value = ""
+        self.value = value
         # self.specified
 
     @property
@@ -1110,8 +1109,29 @@ class Element(Node):
 
     @property
     def attributes(self):
+
         """ Returns a List of an element's attributes """
-        return self.attributes
+        # print('here it is!!!!')
+        # return self.__attributes__
+        # return self.attributes
+        # TODO - does this need a setter?
+        try:
+            # return self.kwargs
+            print("boooooooots::::", [Attr(key.lstrip('_'), value) for key, value in self.kwargs.items()])
+            return [Attr(key.lstrip('_'), value) for key, value in self.kwargs.items()]
+        except Exception as e:
+            print('Error - no tag!', e)
+            return []
+
+    # why bother the user can just access the kwargs directly?
+    # @property
+    # def attributes_(self):
+    #     """ Returns a regular dict of attributes """
+    #     try:
+    #         return self.kwargs
+    #     except Exception as e:
+    #         print('Error - no tag!', e)
+    #         return []
 
     @property
     def innerHTML(self):
@@ -1531,6 +1551,7 @@ class Element(Node):
 
     @property
     def style(self):
+        # print('style GETTER!')
         """ returns the value of the style attribute of an element """
         if self.__style is None:
             self.style = Style()
@@ -1606,23 +1627,21 @@ class Document(Element):
 
     def __init__(self, *args, **kwargs):
         """ init Creates a new Document """
-        # print('setting document !!!!')
         # self.doc = doc
         # self.uri = uri
         # self.documentURI = uri
         # self.documentElement = self
         # self.raw
+        # self.stylesheets = StyleSheetList()
         self.body = ""  # ??
         super().__init__(*args, **kwargs)
         try:
-            # print('setting document !!!!')
             global document
             document = self
         except Exception as e:
             print('failed to set document', e)
 
     def __new__(cls, *args, **kwargs):
-        # print('setting document !!!!')
         instance = super().__new__(cls)
         instance.__init__(*args, **kwargs)
         instance.documentElement = instance
@@ -1630,10 +1649,8 @@ class Document(Element):
         instance.baseURI = domonic.javascript.URL().href
 
         try:
-            # print('setting document baby!!!!')
             global document
             document = instance
-            # print(document)
         except Exception as e:
             print('failed to set document', e)
 
@@ -1708,18 +1725,19 @@ class Document(Element):
 
     @staticmethod
     def createAttribute(name):
-        ''' Creates an attribute node '''
+        """ Creates an attribute node """
         return Attr(name)
 
     @staticmethod
     def createComment(message):
-        '''Creates a Comment node with the specified text'''
+        """ Creates a Comment node with the specified text """
         from domonic.html import comment
         return comment(message)
 
-    # def createDocumentFragment():
-        '''Creates an empty DocumentFragment node'''
-        # return
+    @staticmethod
+    def createDocumentFragment(*args):
+        """ Creates an empty DocumentFragment node if not content passed. I added args as optional to pass content """
+        return DocumentFragment(*args)
 
     @staticmethod
     def createElement(_type):
@@ -2102,6 +2120,179 @@ Console.warn = Console.log
 console = Console
 
 
+class DocumentFragment(Node):
+
+    def __init__(self, *args):
+        self.args = args
+
+    querySelector = Document.querySelector
+    querySelectorAll = Document.querySelectorAll
+    getElementById = Document.getElementById
+    getElementsByTagName = Document.getElementsByTagName
+    _matchElement = Document._matchElement
+    attributes = Element.attributes
+
+    def replaceChildren(self, newChildren):
+        """ Replaces the childNodes of the DocumentFragment object. """
+        self.content.replaceChild(newChildren)
+
+    def __str__(self):
+        return ''.join([str(a) for a in self.args])
+
+
+class Sanitizer():
+
+    def __init__(self, rules=None, *args, **kwargs):
+        """ Creates and returns a Sanitizer object."""
+
+        # casting as object gives us . notation
+        from domonic.javascript import Object
+        self._default_configuration = Object({
+        "allowCustomElements": False,
+        #"allowElements": [],  # elements that the sanitizer should retain in the input.
+        "blockElements": [],  # elements where the sanitizer should remove the elements from the input, but retain their children.
+        "dropElements": [],  # elements that the sanitizer should remove from the input, including its children.
+        #"allowAttributes": [],  # determines whether an attribute (on a given element) should be allowed.
+        "dropAttributes": [],  # determines whether an attribute (on a given element) should be dropped.
+        "allowCustomElements": False,  # determines whether custom elements are to be considered. The default is to drop them. If this option is true, custom elements will still be checked against all other built-in or configured configured checks.
+        "allowComments": False,  # determines whether HTML comments are allowed.
+        "allowElements": ["a", "abbr", "acronym", "address", "area", "article", "aside", "audio", "b", "bdi", "bdo", "bgsound",
+        "big", "blockquote", "body", "br", "button", "canvas", "caption", "center", "cite", "code", "col", "colgroup", "datalist",
+        "dd", "del", "details", "dfn", "dialog", "dir", "div", "dl", "dt", "em", "fieldset", "figcaption", "figure", "font", "footer",
+        "form", "h1", "h2", "h3", "h4", "h5", "h6", "head", "header", "hgroup", "hr", "html", "i", "img", "input", "ins", "kbd", "keygen",
+        "label", "layer", "legend", "li", "link", "listing", "main", "map", "mark", "marquee", "menu", "meta", "meter", "nav", "nobr",
+        "ol", "optgroup", "option", "output", "p", "picture", "popup", "pre", "progress", "q", "rb", "rp", "rt", "rtc", "ruby", "s",
+        "samp", "section", "select", "selectmenu", "small", "source", "span", "strike", "strong", "style", "sub", "summary", "sup",
+        "table", "tbody", "td", "tfoot", "th", "thead", "time", "tr", "track", "tt", "u", "ul", "var", "video", "wbr"],
+        "allowAttributes": {
+            "abbr": ["*"], "accept": ["*"], "accept-charset": ["*"], "accesskey": ["*"], "action": ["*"], "align": ["*"],
+            "alink": ["*"], "allow": ["*"], "allowfullscreen": ["*"], "alt": ["*"], "anchor": ["*"], "archive": ["*"], "as": ["*"],
+            "async": ["*"], "autocapitalize": ["*"], "autocomplete": ["*"], "autocorrect": ["*"], "autofocus": ["*"], "autopictureinpicture": ["*"],
+            "autoplay": ["*"], "axis": ["*"], "background": ["*"], "behavior": ["*"], "bgcolor": ["*"], "border": ["*"], "bordercolor": ["*"],
+            "capture": ["*"], "cellpadding": ["*"], "cellspacing": ["*"], "challenge": ["*"], "char": ["*"], "charoff": ["*"], "charset": ["*"],
+            "checked": ["*"], "cite": ["*"], "class": ["*"], "classid": ["*"], "clear": ["*"], "code": ["*"], "codebase": ["*"], "codetype": ["*"],
+            "color": ["*"], "cols": ["*"], "colspan": ["*"], "compact": ["*"], "content": ["*"], "contenteditable": ["*"], "controls": ["*"],
+            "controlslist": ["*"], "conversiondestination": ["*"], "coords": ["*"], "crossorigin": ["*"], "csp": ["*"], "data": ["*"], "datetime": ["*"],
+            "declare": ["*"], "decoding": ["*"], "default": ["*"], "defer": ["*"], "dir": ["*"], "direction": ["*"], "dirname": ["*"], "disabled": ["*"],
+            "disablepictureinpicture": ["*"], "disableremoteplayback": ["*"], "disallowdocumentaccess": ["*"], "download": ["*"], "draggable": ["*"],
+            "elementtiming": ["*"], "enctype": ["*"], "end": ["*"], "enterkeyhint": ["*"], "event": ["*"], "exportparts": ["*"], "face": ["*"], "for": ["*"],
+            "form": ["*"], "formaction": ["*"], "formenctype": ["*"], "formmethod": ["*"], "formnovalidate": ["*"], "formtarget": ["*"], "frame": ["*"],
+            "frameborder": ["*"], "headers": ["*"], "height": ["*"], "hidden": ["*"], "high": ["*"], "href": ["*"], "hreflang": ["*"], "hreftranslate": ["*"],
+            "hspace": ["*"], "http-equiv": ["*"], "id": ["*"], "imagesizes": ["*"], "imagesrcset": ["*"], "importance": ["*"], "impressiondata": ["*"],
+            "impressionexpiry": ["*"], "incremental": ["*"], "inert": ["*"], "inputmode": ["*"], "integrity": ["*"], "invisible": ["*"], "is": ["*"], "ismap": ["*"],
+            "keytype": ["*"], "kind": ["*"], "label": ["*"], "lang": ["*"], "language": ["*"], "latencyhint": ["*"], "leftmargin": ["*"], "link": ["*"], "list": ["*"],
+            "loading": ["*"], "longdesc": ["*"], "loop": ["*"], "low": ["*"], "lowsrc": ["*"], "manifest": ["*"], "marginheight": ["*"], "marginwidth": ["*"], "max": ["*"],
+            "maxlength": ["*"], "mayscript": ["*"], "media": ["*"], "method": ["*"], "min": ["*"], "minlength": ["*"], "multiple": ["*"], "muted": ["*"], "name": ["*"],
+            "nohref": ["*"], "nomodule": ["*"], "nonce": ["*"], "noresize": ["*"], "noshade": ["*"], "novalidate": ["*"], "nowrap": ["*"], "object": ["*"], "open": ["*"],
+            "optimum": ["*"], "part": ["*"], "pattern": ["*"], "ping": ["*"], "placeholder": ["*"], "playsinline": ["*"], "policy": ["*"], "poster": ["*"], "preload": ["*"],
+            "pseudo": ["*"], "readonly": ["*"], "referrerpolicy": ["*"], "rel": ["*"], "reportingorigin": ["*"], "required": ["*"], "resources": ["*"], "rev": ["*"],
+            "reversed": ["*"], "role": ["*"], "rows": ["*"], "rowspan": ["*"], "rules": ["*"], "sandbox": ["*"], "scheme": ["*"], "scope": ["*"], "scopes": ["*"],
+            "scrollamount": ["*"], "scrolldelay": ["*"], "scrolling": ["*"], "select": ["*"], "selected": ["*"], "shadowroot": ["*"], "shadowrootdelegatesfocus": ["*"],
+            "shape": ["*"], "size": ["*"], "sizes": ["*"], "slot": ["*"], "span": ["*"], "spellcheck": ["*"], "src": ["*"], "srcdoc": ["*"], "srclang": ["*"], "srcset": ["*"],
+            "standby": ["*"], "start": ["*"], "step": ["*"], "style": ["*"], "summary": ["*"], "tabindex": ["*"], "target": ["*"], "text": ["*"], "title": ["*"],
+            "topmargin": ["*"], "translate": ["*"], "truespeed": ["*"], "trusttoken": ["*"], "type": ["*"], "usemap": ["*"], "valign": ["*"], "value": ["*"],
+            "valuetype": ["*"], "version": ["*"], "virtualkeyboardpolicy": ["*"], "vlink": ["*"], "vspace": ["*"], "webkitdirectory": ["*"], "width": ["*"], "wrap": ["*"]
+            }
+        })
+
+        self.config = None
+        if isinstance(rules, dict):
+            self.rules = rules
+            # create a new configuration which is a copy of the default but change it based on the rules object
+            import copy
+            self.config = copy.deepcopy(self._default_configuration)
+            for key, value in self.rules.items():
+                # print('ADDING RULES', key, value)
+                self.config[key] = value
+        else:
+            self.rules = None
+            self.config = self._default_configuration
+
+    def getDefaultConfiguration(self):
+        return self._default_configuration
+
+    def getConfiguration(self):
+        """[return the configuration object]
+
+        Returns:
+            [Object]: [an Object with the users configuration]
+        """
+        return self.config
+
+    def sanitize(self, frag):
+        """ Returns a sanitized DocumentFragment from an input, removing any offending elements or attributes. """
+        if isinstance(frag, str):
+            # parse to html then remove all the bad stuff?? - is a really bad idea. as it goes through eval.
+            frag = domonic.domonic.load(frag)
+
+        isDomNode = False
+        if isinstance(frag, Document):
+            isDomNode = True
+
+        if not isDomNode:
+            newfrag = Document.createDocumentFragment()
+            if isinstance(frag, (tuple, list)):
+                for f in frag:
+                    newfrag.appendChild(f)
+            else:
+                newfrag.appendChild(frag)
+            frag = newfrag
+
+        # TODO "allowCustomElements": # "allowElements": [], # "blockElements": [], # "dropElements": [],  # "allowAttributes": [],
+        # TODO "dropAttributes": # "allowCustomElements": # "allowComments": # "allowElements" # allowAttributes
+
+        for t in self.config["dropElements"]:
+            el = frag.getElementsByTagName(t)
+            el.parentNode.removeChild(el)
+
+        for t in self.config["dropAttributes"]: 
+            for e in self.config["allowElements"]:
+                els = frag.getElementsByTagName(e)
+                if els != False and len(els) > 0:
+                    for el in els:
+                        for each in el.attributes:
+                            if each.name == t:
+                                el.removeAttribute(each.name)
+
+        # print("test" frag.querySelectorAll('span'))
+        # print("test2", frag.getElementsByTagName('span'))
+
+        for e in self.config["allowElements"]:
+            els = frag.getElementsByTagName(e)
+            if els != False and len(els) > 0:
+                for el in els:
+                    # print(el.kwargs, el.attributes, el.__attributes__, type(el.attributes))
+                    for each in el.attributes:
+                        key = each.name
+                        val = each.value
+                        # print(key, val)
+                        allowed_on = self.config["allowAttributes"].get(key)
+                        # print("ALLOWED ON:", key, allowed_on)
+                        if allowed_on == None:
+                            el.removeAttribute(key)
+                            continue
+                        if "*" in allowed_on:
+                            continue
+                        if e not in allowed_on:
+                            el.removeAttribute(key)
+                        # else:
+                        #     print(key + ' is allowed')
+
+        for t in self.config["blockElements"]:
+            el = frag.getElementsByTagName(str(t))
+            # keep the children of the element and add them back to the parent
+            for c in el.childNodes:
+                frag.parentNode.appendChild(c)
+            # remove the element
+            frag.parentNode.removeChild(el)
+
+        # print(type(frag))
+        return frag
+
+    def sanitizeToString(self, frag):
+        """ Returns a sanitized String from an input, removing any offending elements or attributes. """
+        return str(self.sanitize(frag))
+
 
 # document will be set when a Document is created. the last created document is the document that will be used
 # it can also be set manually
@@ -2144,4 +2335,4 @@ class dom(object):  # don't think this class is need now as the package is the '
         # self.webstorage = type('webstorage', (DOM,), {'name':'webstorage'})
 
         pass
-'''
\ No newline at end of file
+'''
