@@ -9,6 +9,7 @@ from domonic.javascript import *
 
 
 class Particle():
+    """ a particle class """
 
     def __init__(self, size=1):
         self.x = 0
@@ -30,8 +31,11 @@ class Particle():
         self._turnToPath = False
         self._bounds = {}
         self.set_bounds({'xMin': 0, 'yMin': 0, 'yMax': 800, 'xMax': 800})
+        # self._bounds = {'top': 0, 'bottom': 0, 'left': 0, 'right': 0}
         self.maxSpeed = Number.MAX_VALUE
 
+        # self.turnToPath(False)
+
     def set_bounds(self, oBounds):
         self._bounds['top'] = oBounds['yMin']
         self._bounds['bottom'] = oBounds['yMax']
@@ -91,6 +95,23 @@ class Particle():
                     self.y = self._bounds['top'] + self.height / 2
                     self.vy *= self.bounce
 
+            # elif self.edges == "none":
+            #     pass
+            # elif self.edges == "destroy":
+            #     self.x = 0
+            #     self.y = 0
+            #     self.vx = 0
+            #     self.vy = 0
+            #     self.rotation = 0
+            # elif self.edges == "explode":
+            #     self.x = 0
+            #     self.y = 0
+            #     self.vx = 0
+            #     self.vy = 0
+            #     self.rotation = 0
+            #     self.vx += Math.random() * self.wander - self.wander / 2
+            #     self.vy += Math.random() * self.wander - self.wander / 2
+
             # else if self.edges == "remove":
             #     if(self.x > self._bounds.right + self.width/2 || self.x<self._bounds.left - self.width/2 ||
             #        self.y > self._bounds.bottom + self.height/2 || self.y<self._bounds.top - self.height/2){
@@ -127,6 +148,13 @@ class Particle3D():
         self.set_bounds({'xMin': 0, 'yMin': 0, 'yMax': 800, 'xMax': 800})
         self.maxSpeed = Number.MAX_VALUE
 
+    def __str__(self):
+        return "Particle3D(x={}, y={}, z={}, width={}, height={}, length={}, rotation={}, vx={}, vy={}, vz={}, damp={}, bounce={}, grav={}, wander={}, k={}, edges={}, drag={}, turnToPath={}, bounds={})".format(
+            self.x, self.y, self.z, self.width, self.height, self.length, self.rotation, self.vx, self.vy, self.vz, self.damp, self.bounce, self.grav, self.wander, self._k, self.edges, self._drag, self._turnToPath, self._bounds)
+
+    # def __repr__(self):
+    #     return self.__str__()
+
     def set_bounds(self, oBounds):
         self._bounds['top'] = oBounds['yMin']
         self._bounds['bottom'] = oBounds['yMax']
