@@ -9,7 +9,7 @@ from domonic.cmd import *
 from domonic.decorators import silence
 
 
-class domonicTestCase(unittest.TestCase):
+class TestCase(unittest.TestCase):
 
     def test_cmd_dir(self):
         files = dir()
