@@ -6,6 +6,7 @@
 
 import time
 import unittest
+from unittest.mock import Mock
 # import requests
 # from mock import patch
 
@@ -849,7 +850,7 @@ class TestCase(unittest.TestCase):
 
 
     # def test_javascript_Node(self):
-        # url = require('url');        
+        # url = require('url');
         # console.log(url.domainToASCII('español.com'))
         # console.log(url.domainToASCII('??.com'))
         # console.log(url.domainToASCII('xn--iñvalid.com'))
@@ -966,7 +967,7 @@ class TestCase(unittest.TestCase):
         '''
         # TODO - make the following work. js sets can have dictionaries in them
         o = {'a': 1, 'b': 2}
-        mySet1.add(o)  # TODO - ok. so we learned something. i nearly never use sets so this is a nice example of difference between python and javascript sets. 
+        mySet1.add(o)  # TODO - ok. so we learned something. i nearly never use sets so this is a nice example of difference between python and javascript sets.
         assert mySet1.size == 4
         assert mySet1.contains(a) == True
 
@@ -991,6 +992,72 @@ class TestCase(unittest.TestCase):
         # logs Set(4) { 1, "some text", {…}, {…} } in Chrome
         '''
 
+    def test_setTimeout(self):
+        """ Test the Global.setTimeout function calls the callback. """
+
+        callback = Mock()
+
+        args = ("hello", "world")
+        kwargs = {"foo": "bar"}
+
+        timer_id = Global.setTimeout(callback, 1000, *args, **kwargs)
+        callback.assert_not_called()
+
+        assert timer_id in Global._Global__timers
+
+        time.sleep(1.5)
+
+        callback.assert_called_once()
+        callback.assert_called_with(*args, **kwargs)
+
+    def test_clearTimeout(self):
+        """ Test that Global.clearTimeout function can cancel a timeout. """
+
+        callback = Mock()
+
+        args = ("hello", "world")
+        kwargs = {"foo": "bar"}
+
+        timer_id = Global.setTimeout(callback, 1000, *args, **kwargs)
+        callback.assert_not_called()
+
+        assert timer_id in Global._Global__timers
+        Global.clearTimeout(timer_id)
+        assert timer_id not in Global._Global__timers
+
+        time.sleep(1.5)
+        callback.assert_not_called()
+
+    def test_timeouts(self):
+        def somefunc():
+            print("hi")
+        someID = Global.setTimeout(somefunc, 2000)
+        Global.clearTimeout(someID)
+        time.sleep(2.5)
+        # nice!
+
+        # from domonic.javascript import setTimeout, clearTimeout
+        # setTimeout(somefunc, 2000)
+        # time.sleep(2.5)
+
+        # from domonic.javascript import setInterval, clearInterval
+        # def somefunc2():
+            # print("testing interval")
+        # interval_id = setInterval(somefunc2, 2000)
+        # time.sleep(10)
+        # clearInterval(interval_id)
+        # time.sleep(4)
+        # print('end')
+
+        # from domonic.javascript import window
+        # intID = window.setInterval(somefunc2, 2000)
+        # time.sleep(10)
+        # clearInterval(intID)  # not clearing brings it back to life!
+
+
+
+
+
     # def test_storage(self):
     #     print("test_storage")
 
@@ -1020,7 +1087,7 @@ class TestCase(unittest.TestCase):
     #     assert myObj2.address == '123 Main St'
     #     assert myObj2.toString() == '{"name": "John", "age": 30, "address": "123 Main St"}'
 
-    
+
     # def test_symbol(self):
     #     print("test_symbol")
     #     symbol = Symbol('x')
@@ -1030,7 +1097,7 @@ class TestCase(unittest.TestCase):
     #     assert symbol.toString() == 'x'
     #     assert symbol.toNumber() == 0
     #     assert symbol.toString(True) == '0'
-        
+
 
 _intID = None
 _results = []
